/*! 
 * \file   kb_R900.c R900 layer              
 *
 * \brief 
 *         This module is layer for communication with various devices 
 *         which are part of the R900 robot. It offers simple interface 
 *         to the user.
 *        
 * \author   Frederic Lambercy (K-Team SA)                               
 *
 * \note     Copyright (C) 2008 K-TEAM SA
 * \bug      none discovered.                                         
 * \todo     nothing.
 */

#include "korebot.h"

//#define KH3_UNDER_16

/*!
 * handles to the various khepera3 devices
 */
static knet_dev_t *dspic;


/*! R900_init initializes the KoreBot library
 * This function needs to be called BEFORE any other functions.
 * 
 * \param none
 *
 * \return A value:
 *       - <0 on error 
 *       - 0 on success 
 *
 */
int R900_init( void )
{
 int rc;

 /* First of all this function initializes the korebot library */
 
 if((rc = kb_init( 0 , NULL )) < 0 )
 {
 	/* Unable to initialize the korebot library */
	KB_ERROR("kb_kh3_init",KB_ERROR_KH3KBINIT);
	return KH3_ERROR_KBINIT;
 }
 
 /* Init gpio module */
 kb_gpio_init();
		 
} 





/*!
 * r900_getcommand gets a command frame from a given R900 device.
 * 
 * Function flow:
 * - a) :  retrieve the first byte which is the frame size from the device
 * - b) :  retrieve the required bytes
 *  
 * Normally an end user don't want to use these function as they are
 * assumed as "low level functions".
 * 
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 * \param out is a pointer to a buffer where the command frame will be stored on.
 *
 * \return A value:
 *       - <0 on error (KH3_ERROR_FRMSZERR, KH3_ERROR_SZFMTERR)
 *       - >=0 on success (returns should be the size of frame) 
 *
 * \remark This function requires that kb_kh3_init has been called
 */
int r900_getcommand( knet_dev_t *hDev, unsigned char *out )
{
 char sizeMsg;
 int rc;

 if( knet_llread( hDev, &sizeMsg, 1 ) == 1 ) 
 {
 	rc = knet_llread( hDev, out, sizeMsg );
	if(rc == sizeMsg)
		return rc;
	else
	{
        	printf("Read 1 sizeMsg = %d\r\n", sizeMsg);

		KB_ERROR("knet_getCommand", KB_ERROR_KH3FRMSZERR, rc, sizeMsg);
		return KH3_ERROR_FRMSZ; 
	}
 }
 else 
 {
        printf("Read 2 sizeMsg = %d\r\n", sizeMsg);

 	 KB_ERROR("knet_getCommand", KB_ERROR_KH3SZFMTERR);
	 return KH3_ERROR_SZFMT;
 }
}


/*!
 * R900_sendcommand sets a command frame to a given R900 device.
 *
 * Normally and end user don't want to use these function as they are
 * assumed as "low level functions".
 *   
 * \param hDev is a handle to an openned knet socket (R900:dsPic).
 * \param in is a pointer to a buffer where the command frame to be sent is stored on.
 *
 * \return A value:
 *       - <0 on error (R900_ERROR_FRMSNDERR)
 *       - >=0 on success (returns should be the size of frame)
 *
 * \remark This function requires that kb_kh3_init has been called
 */
int r900_sendcommand( knet_dev_t *hDev, unsigned char *in )
{
 char sizeMsg;

 /* first byte in the frame is the complete frame size */
 sizeMsg = in[0];
	
 if( knet_llwrite( hDev, in, sizeMsg) == sizeMsg) 
 	return sizeMsg;
 else
 {
	printf("sizeMsg = %d\r\n", sizeMsg);
 	KB_ERROR("knet_sendCommand", KB_ERROR_KH3FRMSNDERR);
	return KH3_ERROR_FRMSND;
 }
}



/*!
 * r900_proximity_ir retrieves an instant IR measure.
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 *
 * \return NULL or a pointer to the IR measure
 */
int r900_proximity_ir(char *outbuf, knet_dev_t *hDev)
{

  int rc , i;
 
  /* Frame format : { Size, Command, Terminator } 
   * where the command can be more than 1 byte */
  char cmd[3] = { 2, 'N', 0};

  if(hDev)
  {
 
 	r900_sendcommand( hDev , cmd );
	rc = r900_getcommand( hDev, outbuf );
	
	return rc;
	
  }
 
  return 0;
 
}

/*!
 * r900_ambiant_ir retrieves an instant IR measure.
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the IR measure.
 *
 */
int r900_ambiant_ir(char *outbuf, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, Terminator }
   * where the command can be more than 1 byte */
  char cmd[3] = { 2, 'O', 0};

  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
	rc = r900_getcommand( hDev, outbuf );

	return rc;
  }

  return 0;

}

/*!
 * r900_battery_Voltage retrieves the actual battery voltage.
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the battery voltage.
 *
 */
int r900_battery_voltage(char *outbuf, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, Terminator }
   * where the command can be more than 1 byte */
  char cmd[3] = { 2, 'V', 0};

  if(hDev)
  {
	r900_sendcommand( hDev , cmd );
	rc = r900_getcommand( hDev, outbuf );
	return rc;
  }

  return 0;

}

/*!
 * r900_Read_TV retrieves if a message of the docking is received.
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the battery voltage.
 *
 */
int r900_Read_TV(char *outbuf, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, Terminator }
   * where the command can be more than 1 byte */
  char cmd[3] = { 2, 'T', 0};

  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
        rc = r900_getcommand( hDev, outbuf );
        return rc;
  }

  return 0;

}

/*!
 * r900_Sound_Event retrieves if a sound have occured.
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the battery voltage.
 *
 */
int r900_Sound_Event(char *outbuf, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, Terminator }
   * where the command can be more than 1 byte */
  char cmd[3] = { 2, 'S', 0};

  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
        rc = r900_getcommand( hDev, outbuf );
        return rc;
  }

  return 0;

}

/*!
 * r900_reset_tstamp resets the absolute timestamp.
 *
 * \param outbuf is a buffer where the data will be stored on.
 *  in this case the only answer to expect is z, which is a ack.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 * 
 *
 * \return NULL or a pointer to the ack.
 *
 */
int r900_reset_tstamp(char *outbuf, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, Terminator }
   * where the command can be more than 1 byte */
  char cmd[3] = { 2, 'Z', 0};

  if(hDev)
  {
	r900_sendcommand( hDev , cmd );
	rc = r900_getcommand( hDev, outbuf );

	return rc;
  }

  return 0;

}

/*!
 * r900_revision retrieves the current OS version/revision
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the revision data
 */
int r900_revision(char *outbuf, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, Terminator }
   * where the command can be more than 1 byte */
  char cmd[3] = { 2, 'B', 0};

  if(hDev)
  {
	r900_sendcommand( hDev , cmd );
	rc = r900_getcommand( hDev, outbuf );

	return rc;
  }
  return 0;

}
/*!
 * r900_setLed Configure the 4 LED on the R900 robot
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param led define which led to set
 * \param value define the state of the LED (0 = off, 1 = on, 2 = toggle)
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the revision data
 */

int r900_setLed(char *outbuf, unsigned char led, unsigned char value, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, arg1, arg2, Terminator }
   * where the command can be more than 1 byte */
  char cmd[5] = { 4, 'L', 0, 0, 0};

  cmd[2] = led;
  cmd[3] = value;

  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
        rc = r900_getcommand( hDev, outbuf );
        return rc;
  }
  return 0;
}




/*!
 * r900_setBackupLight Turn ON or OFF
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param value 0 to turn OFF, 1 to turn ON the light 
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the revision data
 */
int r900_setBackupLight(char *outbuf, unsigned char value , knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, arg1, Terminator }
   * where the command can be more than 1 byte */
  char cmd[4] = { 3, 'E', 0, 0};

  cmd[2] = value;

  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
        rc = r900_getcommand( hDev, outbuf );
        return rc;
  }
  return 0;
}

/*!
 * r900_Enable_modem will turn on or off the GSM modem
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the revision data
 */
int r900_Enable_modem(char *outbuf,unsigned char state, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, Terminator }
   * where the command can be more than 1 byte */
  char cmd[4] = { 3, 'D', 0, 0};

  cmd[2] = state;

  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
        rc = r900_getcommand( hDev, outbuf );
        return rc;
  }
  return 0;
}

/*!
 * r900_Set_Phone_Number will configure the phone number
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the revision data
 */
int r900_Set_Phone_Number(char *outbuf,unsigned char size, char *Buffer, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, Terminator }
   * where the command can be more than 1 byte */
  char cmd[18];
  char i = 0;

  cmd[0] = size + 2;					// Add 2 to send the complete message
  cmd[1] = 'C';
  for(i = 0; i < size; i++)
  {
     cmd[i+2] = *(Buffer+i);
  }
  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
        rc = r900_getcommand( hDev, outbuf );
        return rc;
  }
  return 0;
}

/*!
 * r900_Send_SMS will send a text message through the GSM
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the revision data
 */
int r900_Send_SMS(char *outbuf,unsigned char size, char *Buffer, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, Terminator }
   * where the command can be more than 1 byte */
  char cmd[163];
  char i = 0;

  cmd[0] = size + 2;                                    // Add 2 to send the complete message
  cmd[1] = 'A';
  for(i = 0; i < size; i++)
  {
     cmd[i+2] = *(Buffer+i);
  }
  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
        rc = r900_getcommand( hDev, outbuf );
        return rc;
  }
  return 0;
}



/*!
 * r900_korebot_disable disable the KoreBot, warning this will stop all the running software
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the revision data
 */
int r900_korebot_disable(char *outbuf, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, Terminator }
   * where the command can be more than 1 byte */
  char cmd[3] = { 2, 'K', 0};

  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
        rc = r900_getcommand( hDev, outbuf );
        return rc;
  }
  return 0;
}



/*!
 * r900_koremotor_disable turn ON or turn OFF the supply of the KoreMotorLE
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param value 0 will cut the supply of the KoreMotor, 1 will supply it
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the revision data
 */
int r900_koremotor_disable(char *outbuf,unsigned char value, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, arg1, Terminator }
   * where the command can be more than 1 byte */
  char cmd[4] = { 3, 'P', 0, 0};

  cmd[2] = value;

  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
        rc = r900_getcommand( hDev, outbuf );
        return rc;
  }
  return 0;
}

/*!
 * r900_set_motor move the platform
 *
 * \param outbuf is a buffer where the data will be stored on.
 * \param sense define if the motor must stop = 0, elevate =1 or descend = 2 the platform
 * \param speed define the percentage of the PWM duty cycle
 * \param hDev is a handle to an openned knet socket (Khepera3:dsPic).
 *
 * \return NULL or a pointer to the revision data
 */

int r900_set_motor(char *outbuf, unsigned char sense, unsigned char speed, knet_dev_t *hDev)
{
  int rc;

  /* Frame format : { Size, Command, arg1, arg2, Terminator }
   * where the command can be more than 1 byte */
  char cmd[5] = { 4, 'M', 0, 0, 0};

  cmd[2] = sense;
  cmd[3] = speed;

  if(hDev)
  {
        r900_sendcommand( hDev , cmd );
        rc = r900_getcommand( hDev, outbuf );
        return rc;
  }
  return 0;
}

